@@ -50,7 +50,7 @@ import urllib
 from metrique import query_api, user_api, cube_api
 from metrique.config import Config
 from metrique.utils import json_encode, get_cube
-from metriqueu.utils import jsonhash, utcnow
+from metriqueu.utils import utcnow
 
 # setup default root logger, but remove default StreamHandler (stderr)
 # Handlers will be added upon __init__()
@@ -60,8 +60,9 @@ root_logger = logging.getLogger()
 BASIC_FORMAT = "%(name)s:%(message)s"
 FILETYPES = {'csv': pd.read_csv,
              'json': pd.read_json}
-mongo_re = re.compile('[\.$]')
-space_re = re.compile('\s')
+fields_re = re.compile('[\W]+')
+space_re = re.compile('\s+')
+unda_re = re.compile('_')
 
 
 class BaseCube(MutableSequence):
@@ -113,28 +114,35 @@ class BaseCube(MutableSequence):
         return repr(self.objects)
 
 ####################################################################
+    @property
+    def df(self):
+        if self.objects:
+            return pd.DataFrame(self.objects)
+        else:
+            return pd.DataFrame()
+
     @property
     def objects(self):
         return self._objects
 
     @objects.setter
     def objects(self, objects):
+        # convert from other forms to basic list of dicts
         if objects is None:
             objects = []
-        elif not isinstance(objects, (BaseCube, list, tuple)):
-            _t = type(objects)
-            raise TypeError("container value must be a list; got %s" % _t)
-        elif not all([type(o) is dict for o in objects]):
-            raise TypeError("object values must be dict")
         elif isinstance(objects, pd.DataFrame):
             objects = objects.T.to_dict().values()
-
-        if isinstance(objects, BaseCube):
+        elif isinstance(objects, BaseCube):
             objects = objects.objects
-
-        if objects:
-            objects = self._normalize(objects)
-        self._objects = objects
+        # model check
+        if not isinstance(objects, (BaseCube, list, tuple)):
+            _t = type(objects)
+            raise TypeError("container value must be a list; got %s" % _t)
+        if not all([type(o) is dict for o in objects]):
+            raise TypeError("object values must be dict")
+        if not all([o.get('_oid') is not None for o in objects]):
+            raise ValueError("_oid must be defined for all objs")
+        self._objects = self._normalize(objects)
 
     @objects.deleter
     def objects(self):
@@ -143,31 +151,7 @@ class BaseCube(MutableSequence):
 
     @property
     def oids(self):
-        return self.df['_oid'].tolist()
-
-    @property
-    def ids(self):
-        return self.df['_id'].tolist()
-
-    @property
-    def hashes(self):
-        return self.df['_hash'].tolist()
-
-    @property
-    def starts(self):
-        return self.df['_start'].tolist()
-
-    @property
-    def ends(self):
-        return self.df['_ends'].tolist()
-
-####################################################################
-    @property
-    def df(self):
-        if self.objects:
-            return pd.DataFrame(self.objects)
-        else:
-            return pd.DataFrame()
+        return [o['_oid'] for o in self._objects]
 
 ###################### normalization keys/values #################
     def _normalize(self, objects):
@@ -176,62 +160,40 @@ class BaseCube(MutableSequence):
         don't already have one), and more...
         '''
         start = utcnow()
-        # transpose dataframe's axies before converting to dict
-        objs = []
-        for o in objects:
-            if '_oid' not in o:
-                raise ValueError("_oid not defined for obj(%s)" % o)
-            # these add special meta field default values
-            o = self._obj_hash(o)
+        for i, o in enumerate(objects):
+            # normalize fields (alphanumeric characters only, lowercase)
+            o = self._obj_fields(o)
+            # convert empty strings to None (null)
+            o = self._obj_nones(o)
+            # type conversion; if it looks like a float... it is, etc
+            o = self._obj_types(o)
+            # add object meta data the metriqued requires be set per object
             o = self._obj_end(o)
             o = self._obj_start(o, start)
-            o = self._obj_id(o)
-            # these modify the keys or values directly
-            o = self._obj_fields(o)
-            o = self._obj_nones(o)
-            o = self._obj_types(o)
-            objs.append(o)
-        return objs
+            objects[i] = o
+        return objects
 
-    def _obj_hash(self, obj):
-        o = copy(obj)
-        if '_hash' in obj:
-            del o['_hash']
-        if '_start' in obj:
-            del o['_start']
-        if '_end' in obj:
-            del o['_end']
-        if '_id' in obj:
-            del o['_id']
-        obj['_hash'] = jsonhash(o)
-        return obj
-
-    def _obj_end(self, obj, end=None):
-        end = end or obj.get('_end')
-        obj['_end'] = end
-        return obj
-
-    def _obj_start(self, obj, start=None):
-        start = start or obj.get('_start', start) or utcnow()
-        obj['_start'] = start
-        return obj
-
-    def _obj_id(self, obj):
-        obj['_id'] = jsonhash(obj)
-        return obj
+    def _normalize_fields(self, k):
+        k = k.lower()
+        k = space_re.sub('_', k)
+        k = fields_re.sub('',  k)
+        k = unda_re.sub('_',  k)
+        return k
 
     def _obj_fields(self, obj):
         ''' periods and dollar signs are not allowed! '''
-        # replace spaces, lowercase keys, remove $ and . chars
+        # replace spaces, lowercase keys, remove non-alphanumeric
         # WARNING: only lowers the top level though, at this time!
-        return dict((mongo_re.sub('', space_re.sub('_', k.lower())),
-                     v) for k, v in obj.iteritems())
+        return dict((self._normalize_fields(k), v) for k, v in obj.iteritems())
 
     def _obj_nones(self, obj):
         return dict([(k, None) if v == '' else (k, v) for k, v in obj.items()])
 
     def _obj_types(self, obj, type_map=None):
-        'type_map should be a dict with key:field value:type mapping'
+        '''
+        type_map should be a dict with key:field value:type mapping
+        if not provided, run basic "auto-type" assignment
+        '''
         _type = None
         for f, v in obj.items():
             if type_map:
@@ -251,6 +213,15 @@ class BaseCube(MutableSequence):
                     pass
         return obj
 
+    def _obj_end(self, obj, default=None):
+        obj['_end'] = obj.get('_end', default)
+        return obj
+
+    def _obj_start(self, obj, default=None):
+        _start = obj.get('_start', default)
+        obj['_start'] = _start or utcnow()
+        return obj
+
 
 class BaseClient(BaseCube):
     '''
@@ -325,43 +296,6 @@ class BaseClient(BaseCube):
         # with each others logging.
         self.debug_set()
 
-#################### special cube object handlers #######################
-    def __getitem__(self, name):
-        return self.objects[name]
-
-    def __len__(self):
-        return len(self.objects)
-
-    def __iter__(self):
-        return iter(self.objects)
-
-    def __next__(self):
-        yield next(self)
-
-    def next(self):
-        return self.__next__()
-
-    def __getslice__(self, i, j):
-        return self.objects[i:j]
-
-    def __contains__(self, item):
-        return item in self.objects
-
-    def oids(self):
-        raise NotImplementedError
-
-    def keys(self):
-        return self.objects.keys()
-
-    def items(self):
-        return self.objects.items()
-
-    def __str__(self):
-        return str(self.objects)
-
-    def __repr__(self):
-        return repr(self.objects)
-
 ####################### pandas/hd5 python interface ################
     def load_files(self, path, filetype=None, **kwargs):
         '''
@@ -484,7 +418,7 @@ class BaseClient(BaseCube):
             self.config = Config(config_file=config_file)
             self._config_file = config_file
 
-    def save_uri(self, uri, saveas):
+    def urlretrieve(self, uri, saveas):
         return urllib.urlretrieve(uri, saveas)
 
     def whoami(self, auth=False):
@@ -520,7 +454,6 @@ class HTTPClient(BaseClient):
     cube_register = cube_api.register
     cube_update_role = cube_api.update_role
 
-    cube_activity_import = cube_api.activity_import
     cube_save = cube_api.save
     cube_rename = cube_api.rename
     cube_remove = cube_api.remove
@@ -543,14 +476,6 @@ class HTTPClient(BaseClient):
         self._load_session()
         self._auto_login_attempted = False
 
-    def activity_get(self, ids=None):
-        '''
-        Returns a dictionary of `id: [(when, field, removed, added)]` kv pairs
-        that represent the activity history for the particular ids.
-        '''
-        raise NotImplementedError(
-            'The activity_get method is not implemented in this cube.')
-
     def _build_runner(self, kind, kwargs):
         ''' generic caller for HTTP
             A) POST; use data, not params
@@ -607,9 +532,9 @@ class HTTPClient(BaseClient):
     def get_objects(**kwargs):
         raise NotImplementedError
 
-    def extract(self, **kwargs):
-        objs = self.get_objects(**kwargs)
-        return self.cube_save(objs)
+    def extract(self, *args, **kwargs):
+        self.get_objects(*args, **kwargs)  # default: stores into self.objects
+        return self.cube_save()  # default: saves from self.objects
 
     def get_cmd(self, owner, cube, api_name=None):
         '''
